@@ -4,20 +4,27 @@
 package api
 
 import (
+	"github.com/juju/errors"
 	"github.com/juju/juju/api/base"
 	"github.com/juju/juju/network"
+	"github.com/juju/utils/clock"
+	"gopkg.in/juju/names.v2"
 )
 
 var (
+	CertDir               = &certDir
 	NewWebsocketDialer    = newWebsocketDialer
 	NewWebsocketDialerPtr = &newWebsocketDialer
 	WebsocketDialConfig   = &websocketDialConfig
 	SlideAddressToFront   = slideAddressToFront
 	BestVersion           = bestVersion
 	FacadeVersions        = &facadeVersions
-	NewHTTPClient         = &newHTTPClient
+	ConnectWebsocket      = connectWebsocket
 )
 
+// RPCConnection defines the methods that are called on the rpc.Conn instance.
+type RPCConnection rpcConnection
+
 // SetServerAddress allows changing the URL to the internal API server
 // that AddLocalCharm uses in order to test NotImplementedError.
 func SetServerAddress(c *Client, scheme, addr string) {
@@ -30,38 +37,42 @@ func ServerRoot(c *Client) string {
 	return c.st.serverRoot()
 }
 
-// PatchEnvironTag patches the value of the environment tag.
-// It returns a function that reverts the change.
-func PatchEnvironTag(st *State, envTag string) func() {
-	originalTag := st.environTag
-	st.environTag = envTag
-	return func() {
-		st.environTag = originalTag
-	}
-}
-
 // TestingStateParams is the parameters for NewTestingState, so that you can
 // only set the bits that you acutally want to test.
 type TestingStateParams struct {
 	Address        string
-	EnvironTag     string
+	ModelTag       string
 	APIHostPorts   [][]network.HostPort
 	FacadeVersions map[string][]int
 	ServerScheme   string
 	ServerRoot     string
+	RPCConnection  RPCConnection
+	Clock          clock.Clock
+	Broken         chan struct{}
 }
 
 // NewTestingState creates an api.State object that can be used for testing. It
 // isn't backed onto an actual API server, so actual RPC methods can't be
-// called on it. But it can be used for testing general behavior.
-func NewTestingState(params TestingStateParams) *State {
-	st := &State{
+// called on it. But it can be used for testing general behaviour.
+func NewTestingState(params TestingStateParams) Connection {
+	var modelTag names.ModelTag
+	if params.ModelTag != "" {
+		t, err := names.ParseModelTag(params.ModelTag)
+		if err != nil {
+			panic("invalid model tag")
+		}
+		modelTag = t
+	}
+	st := &state{
+		client:            params.RPCConnection,
+		clock:             params.Clock,
 		addr:              params.Address,
-		environTag:        params.EnvironTag,
+		modelTag:          modelTag,
 		hostPorts:         params.APIHostPorts,
 		facadeVersions:    params.FacadeVersions,
 		serverScheme:      params.ServerScheme,
 		serverRootAddress: params.ServerRoot,
+		broken:            params.Broken,
 	}
 	return st
 }
@@ -97,3 +108,10 @@ func (f *resultCaller) BestAPIVersion() int {
 func (f *resultCaller) RawAPICaller() base.APICaller {
 	return nil
 }
+
+// IsMinVersionError returns true if the given error was caused by the charm
+// having a minjujuversion higher than the juju environment's version.
+func IsMinVersionError(err error) bool {
+	_, ok := errors.Cause(err).(minJujuVersionErr)
+	return ok
+}
