@@ -9,6 +9,7 @@ import (
 	"path"
 	"sort"
 
+	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/set"
 	gc "gopkg.in/check.v1"
@@ -19,6 +20,14 @@ import (
 	"github.com/juju/juju/environs/storage"
 )
 
+// PatchOfficialDataSources is used by tests.
+// We replace one of the urls with the supplied value
+// and prevent the other from being used.
+func PatchOfficialDataSources(s *testing.CleanupSuite, url string) {
+	s.PatchValue(&imagemetadata.DefaultUbuntuBaseURL, "")
+	s.PatchValue(&imagemetadata.DefaultJujuBaseURL, url)
+}
+
 // ParseMetadataFromDir loads ImageMetadata from the specified directory.
 func ParseMetadataFromDir(c *gc.C, metadataDir string) []*imagemetadata.ImageMetadata {
 	stor, err := filestorage.NewFileStorageReader(metadataDir)
@@ -26,9 +35,9 @@ func ParseMetadataFromDir(c *gc.C, metadataDir string) []*imagemetadata.ImageMet
 	return ParseMetadataFromStorage(c, stor)
 }
 
-// ParseMetadataFromStorage loads ImageMetadata from the specified storage reader.
-func ParseMetadataFromStorage(c *gc.C, stor storage.StorageReader) []*imagemetadata.ImageMetadata {
-	source := storage.NewStorageSimpleStreamsDataSource("test storage reader", stor, "images")
+// ParseIndexMetadataFromStorage loads Indices from the specified storage reader.
+func ParseIndexMetadataFromStorage(c *gc.C, stor storage.StorageReader) (*simplestreams.IndexMetadata, simplestreams.DataSource) {
+	source := storage.NewStorageSimpleStreamsDataSource("test storage reader", stor, "images", simplestreams.DEFAULT_CLOUD_DATA, false)
 
 	// Find the simplestreams index file.
 	params := simplestreams.ValueParams{
@@ -45,6 +54,13 @@ func ParseMetadataFromStorage(c *gc.C, stor storage.StorageReader) []*imagemetad
 
 	imageIndexMetadata := indexRef.Indexes["com.ubuntu.cloud:custom"]
 	c.Assert(imageIndexMetadata, gc.NotNil)
+	return imageIndexMetadata, source
+}
+
+// ParseMetadataFromStorage loads ImageMetadata from the specified storage reader.
+func ParseMetadataFromStorage(c *gc.C, stor storage.StorageReader) []*imagemetadata.ImageMetadata {
+	imageIndexMetadata, source := ParseIndexMetadataFromStorage(c, stor)
+	c.Assert(imageIndexMetadata, gc.NotNil)
 
 	// Read the products file contents.
 	r, err := stor.Get(path.Join("images", imageIndexMetadata.ProductsFilePath))
