@@ -9,11 +9,11 @@ import (
 	"strings"
 
 	"github.com/juju/errors"
-	"github.com/juju/names"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/txn"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/names.v2"
 
 	"github.com/juju/juju/state"
 	statetesting "github.com/juju/juju/state/testing"
@@ -24,8 +24,8 @@ type ActionSuite struct {
 	ConnSuite
 	charm             *state.Charm
 	actionlessCharm   *state.Charm
-	service           *state.Service
-	actionlessService *state.Service
+	service           *state.Application
+	actionlessService *state.Application
 	unit              *state.Unit
 	unit2             *state.Unit
 	charmlessUnit     *state.Unit
@@ -47,23 +47,23 @@ func (s *ActionSuite) SetUpTest(c *gc.C) {
 	s.actionlessService = s.AddTestingService(c, "actionless", s.actionlessCharm)
 	c.Assert(err, jc.ErrorIsNil)
 
-	sUrl, _ := s.service.CharmURL()
-	c.Assert(sUrl, gc.NotNil)
-	actionlessSUrl, _ := s.actionlessService.CharmURL()
-	c.Assert(actionlessSUrl, gc.NotNil)
+	sURL, _ := s.service.CharmURL()
+	c.Assert(sURL, gc.NotNil)
+	actionlessSURL, _ := s.actionlessService.CharmURL()
+	c.Assert(actionlessSURL, gc.NotNil)
 
 	s.unit, err = s.service.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.unit.Series(), gc.Equals, "quantal")
 
-	err = s.unit.SetCharmURL(sUrl)
+	err = s.unit.SetCharmURL(sURL)
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.unit2, err = s.service.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.unit2.Series(), gc.Equals, "quantal")
 
-	err = s.unit2.SetCharmURL(sUrl)
+	err = s.unit2.SetCharmURL(sURL)
 	c.Assert(err, jc.ErrorIsNil)
 
 	s.charmlessUnit, err = s.service.AddUnit()
@@ -74,7 +74,7 @@ func (s *ActionSuite) SetUpTest(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.actionlessUnit.Series(), gc.Equals, "quantal")
 
-	err = s.actionlessUnit.SetCharmURL(actionlessSUrl)
+	err = s.actionlessUnit.SetCharmURL(actionlessSURL)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -128,10 +128,10 @@ func (s *ActionSuite) TestAddAction(c *gc.C) {
 		params: map[string]interface{}{
 			"outfile": 5.0,
 		},
-		expectedErr: "JSON validation failed: \\(root\\)\\.outfile : must be of type string, given 5",
+		expectedErr: "validation failed: \\(root\\)\\.outfile : must be of type string, given 5",
 	}} {
 		c.Logf("Test %d: should %s", i, t.should)
-		before := state.NowToTheSecond()
+		before := s.State.NowToTheSecond()
 		later := before.Add(testing.LongWait)
 
 		// Copy params over into empty premade map for comparison later
@@ -161,7 +161,7 @@ func (s *ActionSuite) TestAddAction(c *gc.C) {
 
 			// Enqueued time should be within a reasonable time of the beginning
 			// of the test
-			now := state.NowToTheSecond()
+			now := s.State.NowToTheSecond()
 			c.Check(action.Enqueued(), jc.TimeBetween(before, now))
 			c.Check(action.Enqueued(), jc.TimeBetween(before, later))
 			continue
@@ -171,6 +171,128 @@ func (s *ActionSuite) TestAddAction(c *gc.C) {
 	}
 }
 
+func (s *ActionSuite) TestAddActionInsertsDefaults(c *gc.C) {
+	units := make(map[string]*state.Unit)
+	schemas := map[string]string{
+		"simple": `
+act:
+  params:
+    val:
+      type: string
+      default: somestr
+`[1:],
+		"complicated": `
+act:
+  params:
+    val:
+      type: object
+      properties:
+        foo:
+          type: string
+        bar:
+          type: object
+          properties:
+            baz:
+              type: string
+              default: woz
+`[1:],
+		"none": `
+act:
+  params:
+    val:
+      type: string
+`[1:]}
+
+	// Prepare the units for this test
+	makeUnits(c, s, units, schemas)
+
+	for i, t := range []struct {
+		should         string
+		params         map[string]interface{}
+		schema         string
+		expectedParams map[string]interface{}
+	}{{
+		should:         "do nothing with no defaults",
+		params:         map[string]interface{}{},
+		schema:         "none",
+		expectedParams: map[string]interface{}{},
+	}, {
+		should: "insert a simple default value",
+		params: map[string]interface{}{"foo": "bar"},
+		schema: "simple",
+		expectedParams: map[string]interface{}{
+			"foo": "bar",
+			"val": "somestr",
+		},
+	}, {
+		should: "insert a default value when an empty map is passed",
+		params: map[string]interface{}{},
+		schema: "simple",
+		expectedParams: map[string]interface{}{
+			"val": "somestr",
+		},
+	}, {
+		should: "insert a default value when a nil map is passed",
+		params: nil,
+		schema: "simple",
+		expectedParams: map[string]interface{}{
+			"val": "somestr",
+		},
+	}, {
+		should: "insert a nested default value",
+		params: map[string]interface{}{"foo": "bar"},
+		schema: "complicated",
+		expectedParams: map[string]interface{}{
+			"foo": "bar",
+			"val": map[string]interface{}{
+				"bar": map[string]interface{}{
+					"baz": "woz",
+				}}},
+	}} {
+		c.Logf("test %d: should %s", i, t.should)
+		u := units[t.schema]
+		// Note that AddAction will only result in errors in the case
+		// of malformed schemas, and schema objects can only be
+		// created from valid schemas.  The error handling for this
+		// is tested in the gojsonschema package.
+		action, err := u.AddAction("act", t.params)
+		c.Assert(err, jc.ErrorIsNil)
+		c.Check(action.Parameters(), jc.DeepEquals, t.expectedParams)
+	}
+}
+
+// makeUnits prepares units with given Action schemas
+func makeUnits(c *gc.C, s *ActionSuite, units map[string]*state.Unit, schemas map[string]string) {
+	// A few dummy charms that haven't been used yet
+	freeCharms := map[string]string{
+		"simple":      "mysql",
+		"complicated": "mysql-alternative",
+		"none":        "wordpress",
+	}
+
+	for name, schema := range schemas {
+		svcName := name + "-defaults-service"
+
+		// Add a testing service
+		ch := s.AddActionsCharm(c, freeCharms[name], schema, 1)
+		svc := s.AddTestingService(c, svcName, ch)
+
+		// Get its charm URL
+		sURL, _ := svc.CharmURL()
+		c.Assert(sURL, gc.NotNil)
+
+		// Add a unit
+		var err error
+		u, err := svc.AddUnit()
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(u.Series(), gc.Equals, "quantal")
+		err = u.SetCharmURL(sURL)
+		c.Assert(err, jc.ErrorIsNil)
+
+		units[name] = u
+	}
+}
+
 func (s *ActionSuite) TestEnqueueActionRequiresName(c *gc.C) {
 	name := ""
 
@@ -371,6 +493,32 @@ func (s *ActionSuite) TestFindActionTagsByPrefix(c *gc.C) {
 	}
 }
 
+func (s *ActionSuite) TestFindActionsByName(c *gc.C) {
+	actions := []struct {
+		Name       string
+		Parameters map[string]interface{}
+	}{
+		{Name: "action-1", Parameters: map[string]interface{}{}},
+		{Name: "fake", Parameters: map[string]interface{}{"yeah": true, "take": nil}},
+		{Name: "action-1", Parameters: map[string]interface{}{"yeah": true, "take": nil}},
+		{Name: "action-9", Parameters: map[string]interface{}{"district": 9}},
+		{Name: "blarney", Parameters: map[string]interface{}{"conversation": []string{"what", "now"}}},
+	}
+
+	for _, action := range actions {
+		_, err := s.State.EnqueueAction(s.unit.Tag(), action.Name, action.Parameters)
+		c.Assert(err, gc.Equals, nil)
+	}
+
+	results, err := s.State.FindActionsByName("action-1")
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(len(results), gc.Equals, 2)
+	for _, result := range results {
+		c.Check(result.Name(), gc.Equals, "action-1")
+	}
+}
+
 func (s *ActionSuite) TestActionsWatcherEmitsInitialChanges(c *gc.C) {
 	// LP-1391914 :: idPrefixWatcher fails watcher contract to send
 	// initial Change event
@@ -409,7 +557,7 @@ func (s *ActionSuite) TestActionsWatcherEmitsInitialChanges(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	// per contract, there should be at minimum an initial empty Change() result
-	wc.AssertChange()
+	wc.AssertChangeMaybeIncluding(expectActionIds(a1, a2)...)
 	wc.AssertNoChange()
 }
 
@@ -487,13 +635,15 @@ func (s *ActionSuite) TestMergeIds(c *gc.C) {
 		{changes: "a0,a1,a2", adds: "a1,a4,a5", removes: "a1,a3", expected: "a0,a2,a4,a5"},
 	}
 
+	prefix := state.DocID(s.State, "")
+
 	for ix, test := range tests {
-		updates := mapify(test.adds, test.removes)
-		changes := sliceify(test.changes)
-		expected := sliceify(test.expected)
+		updates := mapify(prefix, test.adds, test.removes)
+		changes := sliceify("", test.changes)
+		expected := sliceify("", test.expected)
 
 		c.Log(fmt.Sprintf("test number %d %#v", ix, test))
-		err := state.WatcherMergeIds(s.State, &changes, updates)
+		err := state.WatcherMergeIds(s.State, &changes, updates, state.ActionNotificationIdToActionId)
 		c.Assert(err, jc.ErrorIsNil)
 		c.Assert(changes, jc.SameContents, expected)
 	}
@@ -502,28 +652,19 @@ func (s *ActionSuite) TestMergeIds(c *gc.C) {
 func (s *ActionSuite) TestMergeIdsErrors(c *gc.C) {
 
 	var tests = []struct {
-		ok   bool
 		name string
 		key  interface{}
 	}{
-		{ok: false, name: "bool", key: true},
-		{ok: false, name: "int", key: 0},
-		{ok: false, name: "chan string", key: make(chan string)},
-
-		{ok: true, name: "string", key: ""},
+		{name: "bool", key: true},
+		{name: "int", key: 0},
+		{name: "chan string", key: make(chan string)},
 	}
 
 	for _, test := range tests {
 		changes, updates := []string{}, map[interface{}]bool{}
-
 		updates[test.key] = true
-		err := state.WatcherMergeIds(s.State, &changes, updates)
-
-		if test.ok {
-			c.Assert(err, jc.ErrorIsNil)
-		} else {
-			c.Assert(err, gc.ErrorMatches, "id is not of type string, got "+test.name)
-		}
+		err := state.WatcherMergeIds(s.State, &changes, updates, state.ActionNotificationIdToActionId)
+		c.Assert(err, gc.ErrorMatches, "id is not of type string, got "+test.name)
 	}
 }
 
@@ -657,8 +798,8 @@ func (s *ActionSuite) TestActionStatusWatcher(c *gc.C) {
 	watchCancelledOrCompleted.AssertChange()
 	watchCancelledOrCompleted.AssertNoChange()
 
-	expect := map[state.ActionStatus][]*state.Action{}
-	all := []*state.Action{}
+	expect := map[state.ActionStatus][]state.Action{}
+	all := []state.Action{}
 	for _, tcase := range testCase {
 		a, err := tcase.receiver.AddAction(tcase.name, nil)
 		c.Assert(err, jc.ErrorIsNil)
@@ -686,7 +827,7 @@ func (s *ActionSuite) TestActionStatusWatcher(c *gc.C) {
 	watchCancelledOrCompleted.AssertNoChange()
 }
 
-func expectActionIds(actions ...*state.Action) []string {
+func expectActionIds(actions ...state.Action) []string {
 	ids := make([]string, len(actions))
 	for i, action := range actions {
 		ids[i] = action.Id()
@@ -698,12 +839,12 @@ func expectActionIds(actions ...*state.Action) []string {
 // easier. It combines two comma delimited strings representing
 // additions and removals and turns it into the map[interface{}]bool
 // format needed
-func mapify(adds, removes string) map[interface{}]bool {
+func mapify(prefix, adds, removes string) map[interface{}]bool {
 	m := map[interface{}]bool{}
-	for _, v := range sliceify(adds) {
+	for _, v := range sliceify(prefix, adds) {
 		m[v] = true
 	}
-	for _, v := range sliceify(removes) {
+	for _, v := range sliceify(prefix, removes) {
 		m[v] = false
 	}
 	return m
@@ -711,7 +852,7 @@ func mapify(adds, removes string) map[interface{}]bool {
 
 // sliceify turns a comma separated list of strings into a slice
 // trimming white space and excluding empty strings.
-func sliceify(csvlist string) []string {
+func sliceify(prefix, csvlist string) []string {
 	slice := []string{}
 	if csvlist == "" {
 		return slice
@@ -719,7 +860,7 @@ func sliceify(csvlist string) []string {
 	for _, entry := range strings.Split(csvlist, ",") {
 		clean := strings.TrimSpace(entry)
 		if clean != "" {
-			slice = append(slice, clean)
+			slice = append(slice, prefix+clean)
 		}
 	}
 	return slice
@@ -734,15 +875,16 @@ type mockAR struct {
 
 var _ state.ActionReceiver = (*mockAR)(nil)
 
-func (r mockAR) AddAction(name string, payload map[string]interface{}) (*state.Action, error) {
+func (r mockAR) AddAction(name string, payload map[string]interface{}) (state.Action, error) {
 	return nil, nil
 }
-func (r mockAR) CancelAction(*state.Action) (*state.Action, error) { return nil, nil }
-func (r mockAR) WatchActionNotifications() state.StringsWatcher    { return nil }
-func (r mockAR) Actions() ([]*state.Action, error)                 { return nil, nil }
-func (r mockAR) CompletedActions() ([]*state.Action, error)        { return nil, nil }
-func (r mockAR) PendingActions() ([]*state.Action, error)          { return nil, nil }
-func (r mockAR) Tag() names.Tag                                    { return names.NewUnitTag(r.id) }
+func (r mockAR) CancelAction(state.Action) (state.Action, error) { return nil, nil }
+func (r mockAR) WatchActionNotifications() state.StringsWatcher  { return nil }
+func (r mockAR) Actions() ([]state.Action, error)                { return nil, nil }
+func (r mockAR) CompletedActions() ([]state.Action, error)       { return nil, nil }
+func (r mockAR) PendingActions() ([]state.Action, error)         { return nil, nil }
+func (r mockAR) RunningActions() ([]state.Action, error)         { return nil, nil }
+func (r mockAR) Tag() names.Tag                                  { return names.NewUnitTag(r.id) }
 
 // TestMock verifies the mock UUID generator works as expected.
 func (s *ActionSuite) TestMock(c *gc.C) {
