@@ -5,14 +5,14 @@ package operation
 
 import (
 	"fmt"
-	"time"
 
 	"github.com/juju/errors"
-	"gopkg.in/juju/charm.v5/hooks"
+	"gopkg.in/juju/charm.v6-unstable/hooks"
 
-	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/status"
 	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/runner"
+	"github.com/juju/juju/worker/uniter/runner/context"
 	"github.com/juju/juju/worker/uniter/runner/jujuc"
 )
 
@@ -55,6 +55,10 @@ func (rh *runHook) Prepare(state State) (*State, error) {
 	if err != nil {
 		return nil, err
 	}
+	err = rnr.Context().Prepare()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
 	rh.name = name
 	rh.runner = rnr
 
@@ -90,13 +94,13 @@ func (rh *runHook) Execute(state State) (*State, error) {
 	err := rh.runner.RunHook(rh.name)
 	cause := errors.Cause(err)
 	switch {
-	case runner.IsMissingHookError(cause):
+	case context.IsMissingHookError(cause):
 		ranHook = false
 		err = nil
-	case cause == runner.ErrRequeueAndReboot:
+	case cause == context.ErrRequeueAndReboot:
 		step = Queued
 		fallthrough
-	case cause == runner.ErrReboot:
+	case cause == context.ErrReboot:
 		err = ErrNeedsReboot
 	case err == nil:
 	default:
@@ -117,7 +121,6 @@ func (rh *runHook) Execute(state State) (*State, error) {
 	if hasRunStatusSet, afterHookErr = rh.afterHook(state); afterHookErr != nil {
 		return nil, afterHookErr
 	}
-
 	return stateChange{
 		Kind:            RunHook,
 		Step:            step,
@@ -131,12 +134,12 @@ func (rh *runHook) beforeHook() error {
 	switch rh.info.Kind {
 	case hooks.Install:
 		err = rh.runner.Context().SetUnitStatus(jujuc.StatusInfo{
-			Status: string(params.StatusMaintenance),
-			Info:   "installing charm software",
+			Status: string(status.Maintenance),
+			Info:   status.MessageInstallingCharm,
 		})
 	case hooks.Stop:
 		err = rh.runner.Context().SetUnitStatus(jujuc.StatusInfo{
-			Status: string(params.StatusMaintenance),
+			Status: string(status.Maintenance),
 			Info:   "cleaning up prior to charm deletion",
 		})
 	}
@@ -158,16 +161,17 @@ func (rh *runHook) afterHook(state State) (bool, error) {
 	case hooks.Stop:
 		// Charm is no longer of this world.
 		err = rh.runner.Context().SetUnitStatus(jujuc.StatusInfo{
-			Status: string(params.StatusTerminated),
+			Status: string(status.Terminated),
 		})
 	case hooks.Start:
 		if hasRunStatusSet {
 			break
 		}
+		logger.Debugf("unit %v has started but has not yet set status", ctx.UnitName())
 		// We've finished the start hook and the charm has not updated its
 		// own status so we'll set it to unknown.
 		err = rh.runner.Context().SetUnitStatus(jujuc.StatusInfo{
-			Status: string(params.StatusUnknown),
+			Status: string(status.Unknown),
 		})
 	}
 	if err != nil {
@@ -210,12 +214,12 @@ func (rh *runHook) Commit(state State) (*State, error) {
 	newState := change.apply(state)
 
 	switch rh.info.Kind {
+	case hooks.Install:
+		newState.Installed = true
 	case hooks.Start:
 		newState.Started = true
 	case hooks.Stop:
 		newState.Stopped = true
-	case hooks.CollectMetrics:
-		newState.CollectMetricsTime = time.Now().Unix()
 	}
 
 	return newState, nil
